@@ -74,7 +74,7 @@ make_local_tmpdir (const char *dirname)
 		}
 	}
 
-#if defined (HAVE_UTIME_H) || defined (HAVE_SYS_UTIME_H)
+#if (defined (HAVE_UTIME_H) || defined (HAVE_SYS_UTIME_H)) && !defined (G_OS_WIN32)
 	{ /* Hide some information ( apparently ) */
 		struct utimbuf utb;
 		memset (&utb, 0, sizeof (utb));
@@ -364,11 +364,27 @@ link_protocol_is_local_ipv46 (const LinkProtocolInfo *proto,
 		if ((af != AF_INET6) && (af != AF_INET))
 			continue;
 
+		/*
+		 * We set the local_addr port number to the same value as
+		 * in the incoming_addr since the port number does not
+		 * affect testing whether the IP address is local or not.
+		 * On some machines, access() fills in the port value so
+		 * saddr will have a non-zero port value, and calling
+		 * getaddrinfo() on "localhost" will have 0 for the port
+		 * value.  So doing this ensures the test will not fail
+		 * if the port numbers are different.  Note that this code
+		 * is protected by the I/O thread so this is handled
+		 * synchronously, so it is safe to modify the static
+		 * local_addr value.
+		 */
 		if (proto->family == AF_INET) {
 			if (af == AF_INET) {
-				if (!memcmp ((struct sockaddr_in *)result->ai_addr,
-					     (struct sockaddr_in *)saddr,
-					     result->ai_addrlen)) {
+				struct sockaddr_in * localaddr;
+				struct sockaddr_in * incomingaddr;
+				localaddr = (struct sockaddr_in *)result->ai_addr;
+				incomingaddr = (struct sockaddr_in *)saddr;
+				localaddr->sin_port = incomingaddr->sin_port;
+				if (!memcmp (localaddr, incomingaddr, result->ai_addrlen)) {
 #ifdef LOCAL_DEBUG
 					g_warning ("local ipv4 address");
 #endif
@@ -378,9 +394,12 @@ link_protocol_is_local_ipv46 (const LinkProtocolInfo *proto,
 		}
 		else {
 			if (af == AF_INET6) {
-				if (!memcmp ((struct sockaddr_in6 *)result->ai_addr,
-					     (struct sockaddr_in6 *)saddr,
-					     result->ai_addrlen)) {
+				struct sockaddr_in6 * localaddr;
+				struct sockaddr_in6 * incomingaddr;
+				localaddr = (struct sockaddr_in6 *)result->ai_addr;
+				incomingaddr = (struct sockaddr_in6 *)saddr;
+				localaddr->sin6_port = incomingaddr->sin6_port;
+				if (!memcmp (localaddr, incomingaddr, result->ai_addrlen)) {
 #ifdef LOCAL_DEBUG
 					g_warning ("local ipv6 address");
 #endif
@@ -795,8 +814,10 @@ link_protocol_get_sockinfo_ipv4 (const LinkProtocolInfo  *proto,
 		hname = host->h_name;
 	}
 #ifdef G_OS_WIN32
-	{
-		/* Make sure looking up that name works */
+	/* Make sure looking up that name works */
+	if (strcmp (hname, "localhost") == 0)
+		hname = "127.0.0.1"; /* Don't take any risk */
+	else {
 		char *hname_copy = g_strdup (hname);
 		host = gethostbyname (hname_copy);
 		
@@ -1138,7 +1159,7 @@ link_protocol_destroy_addr (const LinkProtocolInfo *proto,
  *
  * Return Value: an array of #LinkProtocolInfo structures.
  */
-LinkProtocolInfo * const
+LinkProtocolInfo *
 link_protocol_all (void)
 {
 	return static_link_protocols;
@@ -1153,7 +1174,7 @@ link_protocol_all (void)
  * Return Value: a pointer to a valid #LinkProtocolInfo structure if 
  *               the protocol is supported by linc, NULL otherwise.
  */
-LinkProtocolInfo * const
+LinkProtocolInfo *
 link_protocol_find (const char *name)
 {
 	int i;
@@ -1175,7 +1196,7 @@ link_protocol_find (const char *name)
  * Return Value: a pointer to a valid #LinkProtocolInfo structure if
  *               the protocol is supported by linc, NULL otherwise.
  */
-LinkProtocolInfo * const
+LinkProtocolInfo * 
 link_protocol_find_num (const int family)
 {
 	int i;
